/******************************************************************************

 TEMPORIZADOR CHOC1

ACIONA SADAS A CADA 3 HORAS POR 1 SEGUNDO (OU MAIS) PARA ATIVAR ENTRADA DE AR
EM CHOCADEIRAS
C/ PIC12F675
AUTOR: CLAUDIO LRIOS
DATA: 13/03/2015
PINAGEM DO PIC 12F675:
1-+5V
2- XTAL (4MHZ)
3- XTAL
4- INTERRUPTOR ON/OFF AO GND, COM RESISTOR DE 10K AO VCC (PULL UP)
5- LED AO GND VIA RESISTOR DE 470 OHMS E OUTRO RESISTOR DE 470 AO GND OU VCC
(JUMPER DE ESCOLHA 29/19 DIAS)
6- SADA 1
7- SADA 2 (OPCIONAL)
8- GND
OBS. DEVER SER PREVISTO UMA FONTE DE ALIMENTAO QUE ALIMENTE O CIRCUITO
MESMO EM CASO DE FALTA DE ENERGIA ELTRICA NA REDE, POR SE TRATAR DE LONGOS
PERIODOS.
USO OPCIONAL: VIRAR OVOS
USO DIDTICO
USE COMPILADOR CCS C
ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG LARIOS.TECNOLOGIA.WS
******************************************************************************/
#include <12F675.h>
#device adc=8
#use delay(clock=4000000)
#fuses NOWDT,HS, NOCPD, NOPROTECT, NOMCLR, PUT, BROWNOUT
#use fast_io(A)
#zero_ram
//BYTE
#byte GPIO = 0X05
#byte TRISIO = 0X85
//BIT
#bit sw = 0X5.3 //interruptor on/off pino 4
#bit s1 = 0X5.1// sada 1 pino 6
#bit s2 = 0X5.0//sada 2 (opcional) pino 7
#bit led = 0X5.2//sada para led indicador on/off (piscante) pino 5
#bit tris_led =0X85.2
//flag
unsigned int1 f_29d,f_on,f_t,f_s2,f_s1;
//variavis globais
unsigned int32 tempo=0;
unsigned int32 cnt32=0;
unsigned int16 tempac=0;
unsigned int16 dly=0;
unsigned int8 cts1=0,cts2=0;
//definies de funcionamento
#define v_debounce 10000 //valor para debouce da chave on/off
#define ciclo_ac 3 //ciclo (em horas)
#define v_ts1 1 //em segundos
#define v_ts2 1 //em segundos
#define PERIODO_CHOC1 29 //em dias
#define PERIODO_CHOC2 19 //em dias
/******************************************************************************
ROTINA DE INTERRUPO DO TIMER 0 (CONTA 1 SEGUNDO)
******************************************************************************/
#INT_TIMER0
void INT_TIMER0_ISR(){
//gera 1 segundos com metodo de acumulo de erro e pode ser usado
// para cristais com valores quebrados das mais variadas frequencias
// ajuste o timer para o prescaller igual abaixo no main
#define presc 64 // valor do prescaller do timer usado (8 bits)
#define fator (256*presc*4)
#define f_xtal 4000000 //frequencia do cristal usado (mesmo do cabealho)
cnt32+=fator;
if(cnt32>=f_xtal){
f_t=!f_t;//piscar led on/off
cnt32-=f_xtal;//subtrai xtal da contagem
tempo++;//incrementa contador de tempo mximo
if(cts2){ if(--cts2==0){f_s2=0;} }//decrementa registradores de tempo da sada s2
if(cts1){ if(--cts1==0){f_s1=0; f_s2=1;cts2=v_ts2;}}//idem, da sada s1
if(++tempac>=(ciclo_ac*3600)){ //durao de cada ciclo
tempac=0;//reinicia contagem
f_s1=1;//sada s1 acionar
cts1=v_ts1;//carga do tempo que ficar acionado esta sada
}
if(f_29d){ //testa se periodo desejado  19 ou 29 dias
if(tempo>=(PERIODO_CHOC1*24*3600)){//testa se ciclou por 29 dias e para
f_on=0; //para a contagem
}
}
else{
if(tempo>=(PERIODO_CHOC2*24*3600)){//testa se ciclou por 19 dias e para
f_on=0; //para a contagem
}
}
}
}
/******************************************************************************
ROTINA MAIN
******************************************************************************/
void main() {
SETUP_ADC_PORTS(NO_ANALOGS);//desabilita conversor analogico/digital
setup_comparator(NC_NC_NC_NC);//desliga comparadores internos
setup_counters(RTCC_INTERNAL,RTCC_DIV_64);//prescaller divide por 64
setup_timer_1(T1_DISABLED);
setup_vref(FALSE);//desliga voltagem de referencia
GPIO=0;//idem
TRISIO=0b111000;//sentido fixo das portas
enable_interrupts(int_timer0);//liga interrupo do timer 0
disable_interrupts(global);//mantm desabilitada a geral
/******************************************************************************
LOOP PRINCIPAL DE REPETIO
******************************************************************************/
for(;;){
if((!sw)&&(!dly)) {
disable_interrupts(global);
f_on=!f_on;//inverte
tris_led=1;
if(led) {f_29d=1;} else {f_29d=0;} //le jump e acerta flag entre 29 ou 19 dias
tris_led=0;
if(f_on){ tempo=0; tempac=0; cnt32=0; f_t=1; //reinicia contadores
enable_interrupts(global);
}
else{
f_t=0; f_s1=0;f_s2=0;cts2=0;cts1=0;
}
while(!sw) ;//aguarda soltar interruptor on/off
dly=v_debounce;//recarga do valor de debounce
// enable_interrupts(global);
}
if(dly)dly--;//decrementa contador de debounce se positivo
if(f_on){//repassa valores para saidas de led,s1 e s2
led=f_t;
s1=f_s1;
s2=f_s2;
}
else{ gpio=0;}
}//for(;;)
}//void main()

/*============================fim do programa=================================*/
